% Chapter_12_opt_pde.m
%
% Computational Optimization: Success in Practice
% Chapter 12: Sharing Multiple Software Environments
% Section 12.2: Updating and Tuning Framework
% Section 12.3: 
%
% Optimization framework for solving problems constrained by elliptic PDEs:
%
%  [a(x) u_x]_x = f(x),     x in 2D domain
%          u(x) = phi(x),   x at a boundary
%  2D domain - square [0,b] by [0,b]
%
%    * FreeFEM PDE solver is used to solve the forward and adjoint problems
%    * functions a(x) and u(x) are set by analytical expressions to be used
%      then for creating analytic/synthetic measurements
%    * measurements are given by u*(x) which is continuous function
%
% Included:
% - modes:   regular OPT & TEST (for gradient-based approaches)
% - methods (gradient-based):
%          * steepest descent (SD)
%          * quasi-Newton (BFGS)
%          * Conjugate Gradient (CG)
% - stepsize methods:
%          * constant (CONST)
%          * Golden Section (GS) Search
%          * bracketing-Brent toolbox (BB)
% - output: functional value (in file)
%           iterative solutions (in file)
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; clear; tic;

% setting INPUT parameters
params;

% paths for supplemental MATLAB codes
addpath algorithms;                % algorithms for main code
addpath functions;                 % m-functions

% deleting old & creating new optimization storages
[status, cmdout] = rmdir('temp','s');
if ~status
  disp('Cannot remove temp/ directory!'); return;
end
[status, cmdout] = mkdir('temp');
if ~status
  disp('Cannot create temp/ directory!'); return;
else
  fid = fopen('temp/opt_log.dat', 'w'); fclose(fid);
end

% main INItialization
initialize;

% creating synthetic measurements
measurements;

% initializing control vector
a = load('temp/opt_a_ini.dat'); % reading it from file
aHist = [aHist a'];             % saving in history
numCtrls = length(a);           % size of control vector
fprintf('control vector size: %d\n',numCtrls);

% choosing mode
if strcmp(mode,'OPT')
  mode_OPT;
elseif strcmp(mode,'TEST')
  mode_TEST;
else
  disp(['error: Unknown mode ' mode ' is chosen!']); return;
end

% final output
if (exCode==0)
  return;
elseif strcmp(mode,'OPT')
  file_output;
  fprintf(['Terminated in ' num2str(k) ' iteration(s): termination condition #' num2str(exCode) ' satisfied!\n']);
end
fprintf(['We are fully done! CPU elapsed time = ' num2str(toc) 's\n\n']);
