% Chapter_2_min_1D_GoldenSection.m
%
% Computational Optimization: Success in Practice
% Chapter 2: Minimization Approaches for Functions of One Variable
% Section 2.7: Practical Examples
%
% Coding in MATLAB: Golden Section Search Method
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; FontSize = 14;

% function f(x) and search interval [a, b]
fnNum = 1; % function number
xEx = []; % exact solution if known
a = -1; b = 3;
f = @(x) fn_all_functions(fnNum,x);

% method parameters and constants
epsilon = 0.0002;
alpha = (-1+sqrt(5))/2;

% check if parameters are correct
if (epsilon <= 0) || (epsilon >b-a)
  disp('Incorrect epsilon!!!');
  return;
end

% plotting function f(x)
x = a:0.001:b;
figure(1); subplot(1,2,1);
plot(x,f(x),'b-','LineWidth',2.5); box off;
set(gca,'FontSize',FontSize); hold on;

% setting initial values for k, n, d
k = 1; % iteration count
n = 0; % number of function evaluations
d = b-a; % interval length (error)

% 1st approximation for optimal solution (\bar x)
xOpt = (a+b)/2;

% optimization loop
while(d(end) >= epsilon)  % termination criterion
  x1 = alpha*a + (1-alpha)*b;
  x2 = (1-alpha)*a + alpha*b;
  if k == 1
    f1 = f(x1); f2 = f(x2); % two function evaluations
    n = [n n(end)+2];
  else
    if abs(x2-xOpt(end))<abs(x1-xOpt(end))
      f2 = f1; f1 = f(x1); % one function evaluation
    else
      f1 = f2; f2 = f(x2); % one function evaluation
    end
    n = [n n(end)+1];
  end
  
  if (f1 <= f2)
    b = x2;
    xOpt = [xOpt x1];
  else
    a = x1;
    xOpt = [xOpt x2];
  end

  d = [d b-a];

  % visualization (current solution in red)
  subplot(1,2,1); plot([a b], [f(a) f(b)], 'r.', 'MarkerSize', 25); hold on;
  title(['k = ' num2str(k) ' iterations, d = ' num2str(d(end)) ', x* = ' num2str(xOpt(end))],'FontSize',FontSize); hold on;
  % visualization (interval length)
  subplot(1,2,2); semilogy(n,d,'b.-'); box off; hold on;
  xlabel(['function evaluations, n = ' num2str(n(end))],'FontSize',FontSize);
  title('interval length, d (log-scaled)','FontSize',FontSize);
  axis([0 40 1e-4 1e+1]); set(gca,'FontSize',FontSize);
  pause(1);

  k = k + 1;
  
  % visualization (current solution in blue to "disappear")
  subplot(1,2,1); plot([a b], [f(a) f(b)], 'b.', 'MarkerSize', 25); hold on;
end

% visualization (optimal solution at the end)
subplot(1,2,1); plot([a b], [f(a) f(b)], 'r.', 'MarkerSize', 25); hold on;
title(['k = ' num2str(k-1) ' iterations, d = ' num2str(d(end)) ...
    ' < \epsilon = ' num2str(epsilon) ', x* = ' num2str(xOpt(end))],'Color','r','FontSize',FontSize); hold off;

% convergence analysis
fn_convergence(xEx,xOpt);
