% Chapter_9_LV_example.m
%
% Computational Optimization: Success in Practice
% Chapter 9: ODE-based Optimization
% Section 9.6: Dynamics of Biological Systems by Lotka-Volterra Equations
%
% Example: solving Lotka-Volterra ODE system with fixed parameters
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; clear;
FontSize = 14; LineWidth = 2;

% Define initial conditions: time [t0, tf] & x0 = x(t0)
ti = 0; tf = 15;
x0 = [20 20]';

% Define parameters
params = [1 0.01 1 0.02]; % [alpha beta gamma delta]

% Simulate ODE system using ode23
[t,x] = ode23(@(t,x) fn_LV(params,t,x), [ti tf], x0, params);

% Plotting
figure(1);
plot(t,x(:,1),'b-','LineWidth',LineWidth); hold on;
plot(t,x(:,2),'r-','LineWidth',LineWidth); hold off; 
title('Time history: ode23','FontSize',FontSize); box off;
legend('x_1(t)','x_2(t)','Location','North'); legend boxoff; set(gca,'FontSize',FontSize);
xlabel('t','FontSize',FontSize); ylabel('x(t)','FontSize',FontSize);

figure(2);
plot(x(:,1),x(:,2),'b-','LineWidth',LineWidth); hold on;
plot(x(1,1),x(1,2),'r.','MarkerSize',30); hold off; box off;
title('Phase plane plot: ode23','FontSize',FontSize); set(gca,'FontSize',FontSize);
xlabel('x_1(t)','FontSize',FontSize); ylabel('x_2(t)','FontSize',FontSize);

% Simulate ODE system using ode45
[T,X] = ode45(@(t,x) fn_LV(params,t,x), [ti tf], x0, params);

% Plotting: comparison phase plots for ode23 and ode45
figure(3);
plot(x(:,1),x(:,2),'b-',X(:,1),X(:,2),'r-','LineWidth',LineWidth); hold on; box off;
rectangle('Position',[155 50 15 110]); hold off; legend('ode23','ode45'); legend boxoff;
title('Phase plane plot: ode23 vs. ode45','FontSize',FontSize); set(gca,'FontSize',FontSize);
xlabel('x_1(t)','FontSize',FontSize); ylabel('x_2(t)','FontSize',FontSize);

figure(4);
plot(x(:,1),x(:,2),'b.-',X(:,1),X(:,2),'r.-','LineWidth',LineWidth,'MarkerSize',20);
legend('ode23','ode45'); legend boxoff; box off;
title('Phase plane plot: ode23 vs. ode45','FontSize',FontSize); set(gca,'FontSize',FontSize);
xlabel('x_1(t)','FontSize',FontSize); ylabel('x_2(t)','FontSize',FontSize);
axis([155 170 50 160]);

% Phase plots for ode23 for different initial conditions
figure(5); ti = 0; tf = 7.5;
for ic = 20:5:60
  x0 = [ic ic]';
  [t,x] = ode45(@(t,x) fn_LV(params,t,x), [ti tf], x0, params);
  plot(x(:,1),x(:,2),'b-','LineWidth',LineWidth); hold on;
  plot(x(1,1),x(1,2),'r.','MarkerSize',20); hold on;
end
plot([0 50],[0 100],'b.','MarkerSize',20); hold off; box off;
title('Phase plane plot: ode23','FontSize',FontSize); set(gca,'FontSize',FontSize);
xlabel('x_1(t)','FontSize',FontSize); ylabel('x_2(t)','FontSize',FontSize);
